#!/bin/bash

declare -A package_versions_info=(
    ["kysec-common-devel 1.0.1-16.se.10.ky10 aarch64"]=true
    ["kysec-common-devel 1.0.1-16.se.10.ky10 x86_64"]=true
    ["kysec-common-devel 1.0.1-16.se.10.a.ky10 loongarch64"]=true
    ["kyseclog-daemon 1.0.1-16.se.10.ky10 aarch64"]=true
    ["kyseclog-daemon 1.0.1-16.se.10.ky10 x86_64"]=true
    ["kyseclog-daemon 1.0.1-16.se.10.a.ky10 loongarch64"]=true
    ["libkysec-adv 1.0.1-16.se.10.ky10 aarch64"]=true
    ["libkysec-adv 1.0.1-16.se.10.ky10 x86_64"]=true
    ["libkysec-adv 1.0.1-16.se.10.a.ky10 loongarch64"]=true
    ["libkysec-core 1.0.1-16.se.10.ky10 aarch64"]=true
    ["libkysec-core 1.0.1-16.se.10.ky10 x86_64"]=true
    ["libkysec-core 1.0.1-16.se.10.a.ky10 loongarch64"]=true
    # 添加更多版本及其对应架构...
)

# 函数：检查并安装指定包的指定版本（在给定架构上）
install_package_version() {
    local package="$1"
    local desired_version="$2"
    local desired_architecture="$3"

    # 检查软件包是否已存在
    if ! rpm -q "$package" &>/dev/null; then
        echo "软件包 $package 未安装，无需升级"
    else
        # 获取已安装的包的当前版本
        current_version=$(rpm -q --queryformat '%{VERSION}-%{RELEASE}' "$package" 2>/dev/null)
	cmd_req=$?
        # 检查版本和系统架构是否需要更新
        if [[ $cmd_req -eq 0 && "$current_version" < "$desired_version" ]]; then
            echo "当前版本 $current_version 低于预设版本 $desired_version，并且系统架构为 $desired_architecture，开始安装 $package..."
            
            rpm -Uvh  "./${desired_architecture}/${package}-${desired_version}.${desired_architecture}.rpm"

            # 检查安装结果
            if [[ $? -eq 0 ]]; then
                echo "成功安装了 $package 版本 $desired_version"
            else
                echo "安装 $package 版本 $desired_version 失败，请检查错误信息。"
            fi
        elif [[ $cmd_req -eq 0 ]]; then
            echo "当前版本已经是 $package-$current_version ，此版本不受漏洞影响。"
        else
            echo "无法获取 $package 的当前版本信息。"
        fi
    fi
}

# 遍历并处理每个包及其版本与架构信息
for package_entry in "${!package_versions_info[@]}"; do
    package_name=$(echo "$package_entry" | cut -d ' ' -f 1)
    desired_version=$(echo "$package_entry" | cut -d ' ' -f 2)
    desired_architecture=$(echo "$package_entry" | cut -d ' ' -f 3)
    if [[ "$(uname -m)" == "$desired_architecture" ]]; then
        install_package_version "$package_name" "$desired_version" "$desired_architecture"
    fi
done
